function [down_smooth_X down_smooth_Y] = down_smooth(X, col_X_low, col_X_high, Y, col_Y_low, col_Y_high)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Megan Van Welie
% Worcester Polytechnic Institute
% Silicon Valley MQP 2011
% at SRI International
% megan6459@gmail.com
% Co-Author: Steven Chen
%
% This function was written primarily to downsample the electric field data 
% to the resolution of the magnetic field data. Before downsampling the
% outliers are first removed from both input matrices. The matrices are
% then time matched to one another. The matrix with the larger resolution
% is then downsampled based on the ratio of the the lengths of the two
% matrices. Finally, the two matrices are interpolated to have matching
% time columns. 
%
% Inputs:
% X - input matrix
% col_X_low - starting column of X matrix that needs to have outliers removed
% col_X_high - ending column of X matrix that needs to have outliers removed
% Y - input matrix
% col_Y_low - starting column of Y matrix that needs to have outliers removed
% col_Y_high - ending column of Y matrix that needs to have outliers removed 
%
% Outputs:
% down_smooth_X - the X matrix with matching time range and resolution to
% the Y matrix
% down_smooth_Y - the Y matrix with matching time range and resolution to
% the X matrix
%
%
% NOTE: The starting and ending column inputs are used as inputs
% outlier_removal.m that was called within this function.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %Standardize the range inputs to the outlier_removal function
    column_range_X = strcat(int2str(col_X_low),'-',int2str(col_X_high));
    column_range_Y = strcat(int2str(col_Y_low),'-',int2str(col_Y_high));
    
    %Find outlier removal window sizes and the run outlier removal
    window_X = floor(length(X(:,1))*.007);
    outliers_removed_X = outlier_removal(X, window_X, column_range_X, 6);
    window_Y = floor(length(Y(:,1))*.007);
    outliers_removed_Y = outlier_removal(Y, window_Y, column_range_Y, 6);
    
    %Remove NaN's from matrices that had their outliers removed
    outliers_removed_X(any(isnan(outliers_removed_X),2),:) = [];
    outliers_removed_Y(any(isnan(outliers_removed_Y),2),:) = [];
    
    %Time match the two matrices
    outliers_removed_X = outliers_removed_X((outliers_removed_X(:,1)>=outliers_removed_Y(1,1) & outliers_removed_X(:,1)<=outliers_removed_Y(end,1)),:);
    outliers_removed_Y = outliers_removed_Y((outliers_removed_Y(:,1)>=outliers_removed_X(1,1) & outliers_removed_Y(:,1)<=outliers_removed_X(end,1)),:);

    %Determine which matrix is smaller and which is larger
    if(length(outliers_removed_X)>length(outliers_removed_Y))
        large_data = outliers_removed_X;
        small_data = outliers_removed_Y;
    elseif(length(outliers_removed_X)<length(outliers_removed_Y))
        large_data = outliers_removed_Y;
        small_data = outliers_removed_X;
    else
        down_smooth_X = outliers_removed_X;
        down_smooth_Y = outliers_removed_Y;
        return;
    end
     
    %Find the rate at which the larger matrix will be downsampled
    %This is based on the ratios of length of the large matrix to the
    %length of the small matrix.
    [l_rows, l_cols] = size(large_data);
    [s_rows, ~] = size(small_data);
    downsample_rate = floor(l_rows/s_rows);
    
    %Preallocate a matrix to hold the data from the smoothed large matrix
    smoothed_large = zeros(l_rows,l_cols);
    
    %Smooth the larger matrix before downsampling
    for j = 1:l_cols
        smoothed_large(:,j) = smooth(large_data(:,j), downsample_rate);
    end
    
    %Downsample the large matrix to the length of the small matrix
    %If the time columns of either matrix are not unique (have repeating 
    %time values), then break out of function and continue to next orbit.
    if ((length(unique(smoothed_large(:,1))) == length(smoothed_large(:,1))) && (length(unique(small_data(:,1))) == length(small_data(:,1))))
        downsample_large = interp1(smoothed_large(:,1), smoothed_large, small_data(:,1));
    else
        down_smooth_X = [];
        down_smooth_Y = [];
        return;
    end
        
    %Reassign the large and small matrix to correct output values
    if(length(outliers_removed_X)>length(outliers_removed_Y))
        down_smooth_X = downsample_large;
        down_smooth_Y = small_data;
    else
        down_smooth_X = small_data;
        down_smooth_Y = downsample_large;
    end

end