%TEST
function efield_singlebin(orb_data, ef_data, ax_data,orbitname, saveto)
%Written by James Rehberger and Megan Van Weille. March 4, 2011

%This function takes in orbital data, electric field data, spin axis
%coordinate data, the orbit name, and a directory from the toplevel function 
%process_orbit.m, and processes them to make binned statistical models of
%the electric field data from that orbit in GEO coordinates. The data is saved 
%into the inputted directory into seperate files for each orbit. The 
%function can also be used to analyze and examine single orbit behaviour by 
%plotting the corrected electric field data and other values, which can be 
%saved by a command in the toplevel function as well.




%Try Catch statement makes it so if error occurs, the orbit specified is 
%skipped and process_orbit continues on with the next orbit without stopping 
try 
    
%Identifies and sorts imported data into group matrixes.

                        
%pos_matrix (:,1) = Unix Time
%pos_matrix (:,2:4) = Position X, Y, Z GEO 
%pos_matrix (:,5:7) = Velocity X, Y, Z GEO
%pos_matrix (:,8:10) = B model GEI
pos_matrix = [orb_data{:,1}, orb_data{:,4:5}, orb_data{:,11}];

%Makes an array with matlab time
mtime = unixtime2mat2(pos_matrix(:,1));

%converts the B model in pos_matrix into GEO coordinates
pos_matrix(:,8:10) = onera_desp_lib_rotate(pos_matrix(:,8:10), 'gei2geo', mtime);

%sa_matrix (:,1) = Unix Time
%sa_matrix (:,2) = Right Ascention
%sa_matrix (:,3) = Declination
sa_matrix = [ax_data{:,1:3}];


%ef_matrix (:,1) = Unix Time
%ef_matrix (:,2) = Electric Field in DCS2 X-axis
%ef_matrix (:,3) = Electric Field in DCS2 Z-axis
ef_matrix = [ef_data{:,1}, ef_data{:,3}, ef_data{:,2}];

%Interpolates all the files so that they have the same number of datapoints
%at the same time positions. Makes it so that datapoints are on the same
%time range
pos_matrix = pos_matrix(~isnan(pos_matrix(:,2)),:);
ef_matrix = ef_matrix(~isnan(ef_matrix(:,2)),:);
[interp_pos interp_ef] = data_interp(pos_matrix, ef_matrix);
interp_ef = interp_ef(~isnan(interp_ef(:,2)),:);
sa_matrix = sa_matrix(~isnan(sa_matrix(:,2)),:);
[interp_sa interp_ef] = data_interp(sa_matrix, interp_ef);

%Clears unneeded variables to save memory
clearvars pos_matrix sa_matrix ef_matrix

%Replaces all data with rows that have at least 1 NaN with NaN, and then 
%deletes those rows 
    interp_sa(any(isnan(interp_sa),2),:) = NaN;
    interp_pos(any(isnan(interp_pos),2),:) = NaN;
    interp_ef(any(isnan(interp_ef),2),:) = NaN;
    interp_ef(any(isnan(interp_ef),2),:) = [];
 %   interp_sa(any(isnan(
%Erases any elecric field datapoints that does not match time points
%with the positional and spin axis
    interp_ef = interp_ef(interp_ef(:,1)==interp_sa(:,1) & interp_ef(:,1)==interp_pos(:,1),:);
    
%Interpolates the interp_sa to match the size of interp_pos
interp_sa(any(isnan(interp_sa),2),:) = [];
    interp_pos(any(isnan(interp_pos),2),:) = [];
    [interp_sa interp_pos] = data_interp(interp_sa, interp_pos);

%Converts the spin axis Data into GEI coordinates, then to GEO coordiantes
spin_x_gei = cosd((pi/180)*interp_sa(:,3)).*cosd((pi/180)*interp_sa(:,2));
spin_y_gei = cosd((pi/180)*interp_sa(:,3)).*sind((pi/180)*interp_sa(:,2));
spin_z_gei = sind((pi/180)*interp_sa(:,3));
sa_cart_gei = [spin_x_gei, spin_y_gei, spin_z_gei];
mtime = unixtime2mat2(interp_pos(:,1));
sa_cart_geo = onera_desp_lib_rotate(sa_cart_gei, 'gei2geo', mtime);

%% Calculating angle between magnetic field and projection of the magnetic field on the spin plane

%Normalizes the spin axis coordinates
sa_cgeo_norm = normr(sa_cart_geo);

%Converts the B model data into watts from nanowatts and normalizes the
%data
bmod_geo = interp_pos(:,8:10)*1e-9;
bmod_geo_norm = normr(bmod_geo);

%Finds the angle between the magnetic field and the spin axis by taking the
%inverse cosine of the dot product between the magnetic field and the spin
%axis, and then subtracting the value from 90 degrees
theta_tmp = bsxfun(@times,bmod_geo_norm,sa_cgeo_norm);    %dot product
theta_tmp = theta_tmp(:,1) + theta_tmp(:,2) + theta_tmp(:,3);
theta = 90 - acosd(theta_tmp);


%% Calculate VxB electric field contribution

%Takes the dot product of the normalized magnetic field vector and the 
%normalized spin axis, makes a 1 by 3 matrix of that value, multiplies it
%by the normalized spin axis vector, and subtracts that matrix from the
%normalized magnetic field vector to get the Z dcs portion of the Magnetic
%Field
zdcs=normr(bmod_geo_norm-repmat(dot(bmod_geo_norm,sa_cgeo_norm,2),[1 3]).*sa_cgeo_norm);

%satellite velocity in seconds
vel_geo = interp_pos(:,5:7)*1000; 

%Takes the cross product of the velocity and the magnetic field, and then
%takes the dot product of that and zdcs in order to get the negative of the
%generated electric field in volts, and then converts that to millivolts
Ez_VxB = -1*dot(cross(bmod_geo,vel_geo),zdcs,2);
Ez_VxB = Ez_VxB*1000;


 %% Remove electric field contribution offset from electric field data

%Subtracts the generated electric field from the recorded electric field
%to get a corrected matrix
 ef_dcs_z = interp_ef(:,3) - Ez_VxB;
 ef_dcs_z = [interp_ef(:,1) ef_dcs_z];
 
%Cuts off the first 6 and last 7 seconds of the electric field data in
%order to get rid of the faulty data that is recorded from activation and
%deactivation
 index = find(~isnan(ef_dcs_z(:,2)), 1, 'last');
 ef_dcs_z(ef_dcs_z(:,1) >= (ef_dcs_z(index,1)-7),2:3) = NaN;
 index = find(~isnan(ef_dcs_z(:,2)), 1, 'first');
 ef_dcs_z(ef_dcs_z(:,1) <= (ef_dcs_z(index,1)+6),2:3) = NaN;
 
%Removes outling electric field datapoints 
 ef_dcs_z = outlier_removal(ef_dcs_z, 10000, '2', 2);

%Creates an array ranging from -20 to 20, incrementing .01 for each unit
     thetvec=-20:.01:20;
     k=0;
     
%Looks for rows of data that have a theta value that falls within a
%range of theta, then takes the root mean squared of the electric
%field data that are alligned with the qualifying theta values. Also
%records the number of datapoints that fall in each bin. If there are
%no points that fall within the range, the bin's value is NaN
     for thet=thetvec
         k=k+1;
         ind=find(theta>=thet & theta<(thet+.01));
         if ~isempty(ind)
            rmse(k,1)= sqrt(mean(ef_dcs_z(ind,2).^2));
            num(k,1) = length(ind);
         else
             rmse(k,1) = NaN;
             num(k,1) = NaN;
         end
     end

%Saves the binned data, the theta range, and the number of points per bin
save_path = strcat(saveto, 'Single_Orbit_', orbitname);
     save(save_path, 'thetvec','rmse','num');

%Creates values that represent the start time, end time, and label
%increments for plots
    startdate = mtime(1);
    enddate = mtime(end);
    start_time = datestr(startdate);
    end_time = datestr(enddate);
    timetick = linspace(startdate, enddate, 10);

%% Ploting stuff 
 
%Plots the generated theta vs. time
figure;
plot(mtime,theta)
xlabel('Time');
ylabel('Degrees');
title([start_time,' to ', end_time]);
set(gca, 'XTick', timetick);
datetick('x','HH:MM', 'keepticks');
legend('Theta (deg)');

%Plots the corrected electric field, raw electric field, generated electric
%current and theta vs time
figure;
 plot(mtime,interp_ef(:,3), 'r', mtime, ef_dcs_z(:,2), 'b',mtime, -Ez_VxB, 'g', mtime, theta, 'k')
 xlabel('Time');
ylabel('mV/m,   Degrees');
title([start_time,' to ', end_time]);
set(gca, 'XTick', timetick);
datetick('x','HH:MM', 'keepticks');
legend('Corrected Electric Field', 'Unaltered Electric Field', 'Generated Electric Current', 'Theta');
 
%Plots the binned electrical data vs. theta
figure;
plot(thetvec, rmse);
xlabel('theta');
ylabel('mV/m');
title('Electric Field Vs. Theta');

 catch exception
     
%If an error occurs, most likely a memory error, the program will report
%it, indicate which orbit had failed, clear all variables generated, and
%then proceed to the next orbit without stopping the above function
    fprintf([getReport(exception, 'basic', 'hyperlinks', 'off') '\n']);
    cprintf('red', ['Failed at orbit ' orbitname '.\n']);
    clearvars
    return    

end
