function [ef_data_gei ef_data_geo] = get_efield(ef_data, pos_data, sa_data, theta, T, n)
%
%
%       Author: Steven Chen
%               googly.bear@gmail.com
%
%       Edited By:
%               Megan van Welie
%               megan6459@gmail.com
%
%   INPUT:
%       Requires the electric field data matrix downsampled, a theta matrix that is the
%       same size as the ef_data, a T value which will determine the
%       percentage of acceptance, and n value (integer value of original length of efdata 
%       divided by the downsampled sizewhich helps determine sigma, which is used to 
%       calculate the threshold.
%
%       ef_data - outlier removed, smoothed, and downsampled electric field
%       signal (UNIX TIME, Ez, Ey, Ez)
%
%       pos_data -    % Megan van Welie
%       pos_matrix(:,1)       = unix time
%       pos_matrix(:,2:4)     = pos x,y,z (gei)
%       pos_matrix(:,5:7)     = vel x,y,z (gei)
%       pos_matrix(:,8:10)    = pos x,y,z (geo)
%       pos_matrix(:,11:13)   = vel x,y,z (geo)
%       pos_matrix(:,14:16)   = altitude, latitude, longitude
%       pos_matrix(:,17:19)   = b-model x,y,z (gei)
%       pos_matrix(:,20:22)   = b-model x,y,z (geo)
%       pos_matrix(:,23)      = ilat
%       pos_matrix(:,24)      = mlt
%
%       sa_data -     % Megan van Welie
%       spin_axis(:,1)   = unix time
%       spin_axis(:,2:4) = spin axis (GEI)
%       spin_axis(:,5:7) = spin axis (GEO)
% 
%
%       theta - a vector with degrees that have the same time ranges as
%       ef_data and same length
%
%       T - the percentage of acceptance, for now it will be a user input.
%       Value must be less than 1.
%
%       n - integer value of original length of ef_data divided by
%       downsampled ef_data, so that the noise floor sigma can be adjusted
%       accordingly.
%
%
%   OUTPUT:
%
%       ef_data - outputs the same size matrix with NaNs stuffed into the
%       Ez component for where the values do not meet the threshold
%
%
%   DESCRIPTION:
%
%       This function will find values of the Ez component which meet the
%       threshold calculated given inputs theta, T, and n.
%
%       Follows similar principal outlined in outlier_removal, using a
%       Chi-squared distribution
%


%% Remove offset from electric field data

[ef_data(:,2) ef_data(:,4)] = ecurrent_offset(pos_data, ef_data, sa_data);

%% Regain the missing Y component - not implemented because methods not verified
%
% noise_floor = 3/1000; %  Constant of 3mV/m. Determined through statistical analysis 
%                          using function axis.
% 
% sigma = (noise_floor)/sqrt(n);
% threshold = sqrt(-2*(sigma.^2)*log(T));
%
%%Remove the values above threshold
% ef_data(abs(ef_data(:,4)) < threshold, 4) = NaN;
% 
%
% E_2 = ef_data(:,4)./sind(theta); 
% E_2y = -cotd(theta) .* ef_data(:,4);
% 
% %ef_data(:,3) = sqrt((e_mag.^2) -(ef_data(:,2).^2) - (ef_data(:,4).^2));
% ef_data(:,3) = cross(ef_data(:,2), ef_data(:,4));
%
%
%% Find electric field in GEI and GEO from DCS2

ef_total = [ef_data(:,2), ef_data(:,3), ef_data(:,4)];

E_gei = dcs2togeo(ef_total,pos_data(:,17:19),sa_data(:,2:4));

E_geo = dcs2togeo(ef_total,pos_data(:,20:22),sa_data(:,5:7));

ef_data_gei = [ef_data(:,1), E_gei]; %attach time axis to output matrices
ef_data_geo = [ef_data(:,1), E_geo];

end

