function [field_matrix, pos_matrix, sa_matrix, ees_matrix, ies_matrix] = load_data(orb_data, ef_data, mag_data, ees_data, ies_data, sa_data)
% Megan Van Welie
% Worcester Polytechnic Institute
% Silicon Valley MQP 2011
% at SRI International
% megan6459@gmail.com
%
% This function is designed to reorder the input data needed to calculate the Poynting
% flux into matrices that can be used with the MATLAB function "p_process2.m".
% 
% Inputs: six matrices containing positional, electric field, magnetic
% field, electron, ion, and spin axis data. 
%
% The order of the data in the input matrices can be found in the comments
% of the cell below titled "Load the data and reassign to matrices with the
% required information".
%
% Outputs: five matrices containg fields (electric and magnetic) data, positional
% data, spin axis data, electron data, and ion data. The data in these output
% matrices has been unified to work with the function "p_process2.m".
%
% The order of the data in the output matrices can be found in the comments
% of the cell at the end of this document titled "Output final matrices".

%% Load the data and reassign to matrices with the required information
%%%%%%%%%%%%%%PositionalData%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%orb_data{:,1}  = UNIX time
%orb_data{:,2}  = Pos x,y,z(GEI)
%orb_data{:,3}  = Vel x,y,z(GEI)
%orb_data{:,4}  = Pos x,y,z(GEO)
%orb_data{:,5}  = Vel x,y,z(GEO)
%orb_data{:,6}  = Altitude (km) (GEO spherical)
%orb_data{:,7}  = Latitude (deg) (GEO spherical)
%orb_data{:,8}  = Longitude (deg) (GEO spherical)
%orb_data{:,9}  = Invariant Latitude (deg)
%orb_data{:,10} = Magnetic Local Time
%orb_data{:,11} = B Model x,y,z (GEI)
%orb_data{:,12} = B Footprint (GEI mapped to 100 km)
%orb_data{:,13} = Latitude Footprint (geodectic)
%orb_data{:,14} = Longitude Footprint (geodectic)


pos_matrix = [orb_data{:,1:3}, orb_data{:,6:8}, orb_data{:,11}, orb_data{:,9:10},orb_data{:,12}];
%pos_matrix(:,1)       = unix time
%pos_matrix(:,2:4)     = pos x,y,z (gei)
%pos_matrix(:,5:7)     = vel x,y,z (gei)
%pos_matrix(:,8:10)    = altitude(km), latitude (deg), longitude(deg)
%pos_matrix(:,11:13)   = b-model x,y,z (gei)
%pos_matrix(:,14)      = ilat (deg)
%pos_matrix(:,15)      = mlt
%pos_matrix(:,16:18)   = B foot
clearvars orb_data

%%%%%%%%%%%Magnetic Field Data%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%mag_data{:,1}    = UNIX time
%mag_data{:,2:4}  = Delta-B x,y,z (GEI)
%mag_data{:,5:7}  = Total B x,y,z (GEI)
%mag_data{:,8:10} = Delta-B x,y,z (FAC_V)

mag_matrix = [mag_data{:,1:7}];
%mag_matrix(:,1)        = unix time
%mag_matrix(:,2:4)      = delta-b x,y,z (GEI)
%mag_matrix(:,5:7)      = B x,y,z (GEI)
clearvars mag_data

%%%%%%%%%%%Electric Field Data%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%ef_data{:,1}   = UNIX time
%ef_data{:,2}   = E near B (E_z (DCS))
%ef_data{:,3}   = E along V (E_x (DCS))

%Set electric field component to 0 (broken sensor (Y) /noise data (Z))
e_zero = zeros(length(ef_data{:,1}), 1);

ef_matrix = [ef_data{:,1}, ef_data{:,3}, e_zero, ef_data{:,2}];
%ef_matrix(:,1)        = unix time
%ef_matrix(:,2)        = e-field x (DCS2) (along V)
%ef_matrix(:,3)        = e-field y 
%ef_matrix(:,4)        = e-field z (DCS2) (near B)
clearvars ef_data e_zero

%%%%%%%%%Spin Axis Data%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%sa_data{:,1}   = UNIX time
%sa_data{:,2}   = Right Ascenscion Angle (deg)
%sa_data{:,3}   = Declination Angle (deg)

sa_matrix = [sa_data{:,1:3}];
%sa_matrix(:,1) = unix time
%sa_matrix(:,2) = right ascention angle
%sa_matrix(:,3) = declination angle
clearvars sa_data

%%%%%%%%%Electron Data%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%ees_data{:,1}   = UNIX time
%ees_data{:,2}   = Kinetic Energy Flux(Electron)(W/m^2)   

ees_matrix = [double(ees_data{:,1}), double(ees_data{:,2})];
%ees_matrix(:,1) = unix time
%ees_matrix(:,2) = kintetic energy flux (W/m^2)
clearvars ees_data

%%%%%%%%%Ion Data%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%ies_data{:,1}   = UNIX time
%ies_data{:,2}   = Kinetic Energy Flux(Ion)(W/m^2) 
                        
ies_matrix = [double(ies_data{:,1}), double(ies_data{:,2})];
%ies_matrix(:,1) = unix time
%ies_matrix(:,2) = kintetic energy flux (W/m^2)
clearvars ies_data


%% Error Handling for Invalid Data

%If the electric field data is more than 90% NaN's, then do not process
if (sum(~isnan(ef_matrix(:,2)))<(length(ef_matrix(:,2))*.1) ...
        || sum(~isnan(ef_matrix(:,3)))<(length(ef_matrix(:,3))*.1))
    field_matrix = []; 
    pos_matrix = [];  
    ees_matrix = [];  
    ies_matrix = [];  
    sa_matrix = []; 
    return; 
end

%Cut off the first 6 seconds and last 7 seconds of electric field data
%This is to remove outliers caused by the instrument turning on and off.
index = find(~isnan(ef_matrix(:,2)), 1, 'last');
ef_matrix(ef_matrix(:,1) >= (ef_matrix(index,1)-7),2:3) = NaN;
index = find(~isnan(ef_matrix(:,2)), 1, 'first');
ef_matrix(ef_matrix(:,1) <= (ef_matrix(index,1)+6),2:3) = NaN;

%% Convert data to required units

%Convert B model from nanoTesla to Telsa
pos_matrix(:,11:13) = bsxfun(@times, pos_matrix(:,11:13), 1e-9); 

%Convert B FOOT from nanoTesla to Telsa
pos_matrix(:,16:18) = bsxfun(@times, pos_matrix(:,16:18), 1e-9); 

%Convert all magnetic field data from nanoTesla to Telsa
mag_matrix(:,2:7) = bsxfun(@times, mag_matrix(:,2:7), 1e-9); 

%Convert all electric field data from mV/m to V/m
ef_matrix(:,2:4) = bsxfun(@times, ef_matrix(:,2:4), 1e-3);


%% Data conversion from GEI to GEO

%Convert position, velocity, and b-model data from GEI to GEO
m_time_pos  = unixtime2mat2(pos_matrix(:,1));
pos_geo = onera_desp_lib_rotate(pos_matrix(:,2:4),'gei2geo',m_time_pos);
vel_geo = onera_desp_lib_rotate(pos_matrix(:,5:7),'gei2geo',m_time_pos);
bmod_geo = onera_desp_lib_rotate(pos_matrix(:,11:13),'gei2geo',m_time_pos);

pos_matrix_geo = [pos_matrix(:,1:7), pos_geo, vel_geo, pos_matrix(:,8:13), bmod_geo, pos_matrix(:,14:18)];
%pos_matrix(:,1)       = unix time
%pos_matrix(:,2:4)     = pos x,y,z (gei)
%pos_matrix(:,5:7)     = vel x,y,z (gei)
%pos_matrix(:,8:10)    = pos x,y,z (geo)
%pos_matrix(:,11:13)   = vel x,y,z (geo)
%pos_matrix(:,14:16)   = altitude, latitude, longitude
%pos_matrix(:,17:19)   = b-model x,y,z (gei)
%pos_matrix(:,20:22)   = b-model x,y,z (geo)
%pos_matrix(:,23)      = ilat
%pos_matrix(:,24)      = mlt
%pos_matrix(:,25:27)      = BFOOT
clearvars pos_geo vel_geo bmod_geo 

%Convert delta-B and B field data from GEI to GEO
m_time_field = unixtime2mat2(mag_matrix(:,1));
db_geo = onera_desp_lib_rotate(mag_matrix(:,2:4),'gei2geo',m_time_field);
b_geo= onera_desp_lib_rotate(mag_matrix(:,5:7),'gei2geo',m_time_field);

mag_matrix_geo = [mag_matrix(:,1:7),db_geo,b_geo];
%mag_matrix_geo(:,1)      = unix time
%mag_matrix_geo(:,2:4)    = delta-b x,y,z (GEI)
%mag_matrix_geo(:,5:7)    = B (total) x,y,z (GEI)
%mag_matrix_geo(:,8:10)   = delta-b x,y,z (GEO)
%mag_matrix_geo(:,11:13)  = B (total) x,y,z (GEO)
clearvars mag_matrix db_geo b_geo
    

%% Match the times and resolutions of all data

%Downsample the electric field data to match the size of the magnetic field
%data. Data is smoothed and time matched before downsampling.
[~, mag_cols] = size(mag_matrix_geo);
[~, ef_cols] = size(ef_matrix);

[down_smooth_ef down_smooth_mag] = down_smooth(ef_matrix, 2, ef_cols, mag_matrix_geo, 2, mag_cols);
if (isempty(down_smooth_ef) || isempty(down_smooth_mag))
    field_matrix = []; 
    pos_matrix = [];  
    ees_matrix = [];  
    ies_matrix = [];  
    sa_matrix = []; 
    return; 
end
clearvars mag_matrix_geo ef_cols mag_cols

%Find the spin axis (GEI and GEO)
[spin_axis_gei spin_axis_geo] = get_spin_axis(sa_matrix);
spin_axis = [spin_axis_gei(:,1:4), spin_axis_geo(:,2:4)];
%spin_axis(:,1)   = unix time
%spin_axis(:,2:4) = spin axis (GEI)
%spin_axis(:,5:7) = spin axis (GEO)
clearvars sa_matrix spin_axis_gei spin_axis_geo

%Match times and sizes of spin axis to the electric and magnetic field
[interp_sa interp_ef] = data_interp(spin_axis, down_smooth_ef);
[interp_sa interp_mag] = data_interp(interp_sa, down_smooth_mag);
clearvars spin_axis down_smooth_ef down_smooth_mag

interp_mag(any(isnan(interp_sa),2),:) = [];
interp_ef(any(isnan(interp_sa),2),:) = [];
interp_sa(any(isnan(interp_sa),2),:) = [];

%Match times and sizes of positional data to the electric field, magnetic
%field, and spin axis data
[interp_pos interp_sa] = data_interp(pos_matrix_geo, interp_sa);
[interp_pos interp_ef] = data_interp(interp_pos, interp_ef);
[interp_pos interp_mag] = data_interp(interp_pos, interp_mag);
clearvars pos_matrix_geo

interp_mag(any(isnan(interp_pos),2),:) = [];
interp_ef(any(isnan(interp_pos),2),:) = [];
interp_sa(any(isnan(interp_pos),2),:) = [];
interp_pos(any(isnan(interp_pos),2),:) = [];

%% Interpolate to kinetic energy flux
% ees_matrix(any(isnan(ees_matrix),2),:) = [];
% ies_matrix(any(isnan(ees_matrix),2),:) = [];
% pos_matrix(any(isnan(pos_matrix),2),:) = [];
% 
% pos_matrix = pos_matrix((pos_matrix(:,1)>=interp_pos(1,1) & pos_matrix(:,1)<=interp_pos(end,1)),:);
% 
% [interp_ees interp_ies] = data_interp(ees_matrix, ies_matrix);
% [interp_ees interp_pos_ke] = data_interp(interp_ees, pos_matrix);
% [interp_ies interp_pos_ke] = data_interp(interp_ies, interp_pos_ke);
% 
% 
% interp_ees(any(isnan(interp_ees),2),:) = [];
% interp_ies(any(isnan(interp_ies),2),:) = [];
% interp_pos_ke(any(isnan(interp_pos_ke),2),:) = [];


%% Find missing electric field component

theta = get_theta(interp_pos, interp_sa);
n = ceil(length(ef_matrix(:,1))/length(interp_ef(:,1)));

[ef_data_gei ef_data_geo] = get_efield(interp_ef, interp_pos, interp_sa, theta, (1/10), n);
interp_ef_total = [ef_data_gei, ef_data_geo(:,2:4)];
%interp_ef_total(:,1)   = unix time
%interp_ef_total(:,2)   = e-field x (DCS2) (along V)
%interp_ef_total(:,3)   = e-field y (DCS2)
%interp_ef_total(:,4)   = e-field z (DCS2) (near B)
clearvars ef_data_gei ef_data_geo

%% Output final matrices

%Assemble final (interpolated) field matrix
field_matrix = [interp_ef_total, interp_mag(:,2:13)];
%field_matrix(:,1)      = unix time
%field_matrix(:,2:4)    = e-field x,y,z (GEI)
%field_matrix(:,5:7)    = e-field x,y,z (GEO)
%field_matrix(:,8:10)   = delta-b x,y,z (GEI)
%field_matrix(:,11:13)  = B (total) x,y,z (GEI)
%field_matrix(:,14:16)  = delta-b x,y,z (GEO)
%field_matrix(:,17:19)  = B (total) x,y,z (GEO)

pos_matrix = interp_pos;
%pos_matrix(:,1)       = unix time
%pos_matrix(:,2:4)     = pos x,y,z (gei)
%pos_matrix(:,5:7)     = vel x,y,z (gei)
%pos_matrix(:,8:10)    = pos x,y,z (geo)
%pos_matrix(:,11:13)   = vel x,y,z (geo)
%pos_matrix(:,14:16)   = altitude, latitude, longitude
%pos_matrix(:,17:19)   = b-model x,y,z (gei)
%pos_matrix(:,20:22)   = b-model x,y,z (geo)
%pos_matrix(:,23)      = ilat
%pos_matrix(:,24)      = mlt

ees_matrix = ees_matrix;
%ees_matrix(:,1) = unix time
%ees_matrix(:,2) = kintetic energy flux (W/m^2)

ies_matrix = ies_matrix;
%ies_matrix(:,1) = unix time
%ies_matrix(:,2) = kintetic energy flux (W/m^2)

sa_matrix = interp_sa;
%sa_matrix(:,1)   = unix time
%sa_matrix(:,2:4) = spin axis (GEI)
%sa_matrix(:,5:7) = spin axis (GEO)

return 
