function output_signal  = outlier_removal(signal, n, column, order)
%OUTLIER_REMOVAL
%
%
%   AUTHOR: Steven Chen       2/14/11
%           googly.bear@gmail.com
%
%   This function will remove outliers from a sample matrix given the
%   column numbers and the 'n' window the user inputs. The formula follows
%   that which is explained in 'Radar detection of a localized 1.4 Hz 
%   pulsation in auroral plasma,simultaneous with pulsating optical         
%   emissions, during a substorm' written by R. Cosgrove, M. Nicolls, H. 
%   Dahlgren, S. Ranjan, E. Sanchez, and R. Doe. Please refer to this
%   document for more detailed information.
%
%   REFER TO THE MQP PAPER FOR THE CALCULATIONS.
%
%
%
%INPUT
%
%   signal - The matrix with which the has outliers and needs to be cleaned
%   up. The signal may either have NaNs in it or not. The first vector
%   needs to be time so that the indices can then match up.
%   
%   n - The window length that need to be inputted, generally a larger
%   window size is better for faster processing, default is 10000 samples
%
%   column - String input that indicates which columns need to have
%   outliers removed. Currently the implementation can accept ranges and
%   separate columns signified by commas or dashes.
%   EX.   '2-3,5,6-8'
%   This example will go through columns 2,3 then do 5 and do 6,7,8
%
%   *WARNING*
%   Addition of multiple columns into this function will adherently slow
%   the program down. Do not unnecessarily input many columns. At the time
%   of this writing the built in MATLAB median function is the bottleneck.
%
%   
%
%OUTPUT
%
%   output_signal - the modified signal with outliers replaced with NaNs.
%   The same size as the input matrix is outputted.
%
%
%   REVISION HISTORY
%
%   1.0:
%       2/14/11 - 2:26AM
%       First try at program, really late at night... works, but not fully
%       tested
%
%   1.1:
%       2/14/11 - 10:15AM
%       Added error checking for input arguments
%
%   1.2:
%       2/15/11 - 1:35AM
%       Corrected so that the output matrix is the same size as the input
%       matrix, except stuffed with more NaNs due to removal of outliers.
%
%   1.3:
%       2/16/11 - 9:33AM
%       Changed formula such that it takes an n-th term polynomial to fit
%       the data to for outlier removal.

counter = 1; %  Keeps track of where you are in the string
signal_copy = signal; % Creates a copy of the signal so that the output matrix can have same dimensions
%   Preps the input. Clears out all current NaNs in the matrix
signal(any(isnan(signal),2),:) = [];
%   Start a series of checks for the input
    
    %   Check to make sure that the inputted matrix is numeric
    if(~isnumeric(signal))
        cprintf('red', 'Illegal input signal, non-numeric.\n');
        return
    end
    
    %   Check to make sure that n is less than the length of the matrix
    if(n > length(signal))
        cprintf('red', 'Window size n is greater than the length of the sample signal.\n');
        return
    end
        

%   Searches through the column input for commas
    for i = 1:length(column)
        %   Check whether a comma was hit or it was just the end of the string
        %   input
        if(strcmp(column(i), ',') || i == length(column))
            for k = counter:i
                %   Check whether it is a range or just individual values
                %   and convert to the numeric values
                if(strcmp(column(k), '-'))
                    column_start = str2double(column(counter:k-1));
                    column_end = str2double(column(k+1:i));
                    %   Error checking for if the values are numbers
                        if(~isnumeric(column_start) || ~isnumeric(column_end))
                            cprintf('red','Non-numeric ranges for columns.\n');
                            return
                        end
                    %   Error checking for if the matrix column exists
                        if(column_start > size(signal,2) || column_end > size(signal,2))
                            cprintf('red', 'Invalid column range, exceeds matrix dimensions.\n');
                            return
                        end
                    break;
                else
                    %   Single range, make loop go through once
                    column_start = str2double(column(counter:i));
                    column_end = str2double(column(counter:i));
                    if(~isnumeric(column_start) || ~isnumeric(column_end))
                        cprintf('red','Non-numeric ranges for columns.\n');
                        return
                    end    
                    
                    %   Error checking for if the matrix column exists
                    if(column_start > size(signal,2) || column_end > size(signal,2))
                        cprintf('red', 'Invalid column range, exceeds matrix dimensions.\n');
                        return
                    end
                end
            end
            
            %   Keeps track of where in the string to start from next
            %   iteration
            counter = i+1;
            
            
            %   Goes through the column range and operates on each column
            %   respectively
            for j = column_start:column_end
                %   Firstly get rid of all the NaNs in the matrix
                bad_indices = zeros(length(signal(:,j)),1);
                
                %   Catch if the window added with the current value
                %   exceeds the end of the matrix length, if it does then
                %   change the size of the window accordingly.
                for y = 1:n:length(signal(:,j))
                    if(y+n >= length(signal(:,j)))
                        %n = length(signal(:,j)) - y;
                        y = length(signal(:,j)) - n; 
                    end
                    
                    %   The core of the program to calculate whether or not
                    %   the value exceed the threshold value of x
                    
                    z = (signal(y:y+n,1) - mean(signal(y:y+n,1)))./std(signal(y:y+n,1));
                    zfit = linspace(z(1),z(end),n+1);
                    pz = polyfit(z,signal(y:y+n,j), order);
                    yfit = polyval(pz,zfit);
                    
                    V_sub_mean = signal(y:y+n,j)-yfit';
                    V_squared = V_sub_mean.^2;
                    V_median = median(V_squared);
                    V_sigma_squared = V_median/(2*log(2));
                    x = -log(1-(1/2)^(1/n))*2*V_sigma_squared;
                    bad_index = V_squared > x;
                    
                    %   Replace the bad signals values with NaNs
                    bad_indices(y:y+n) = bad_index;
                end
                %   Finds the indices where the altered signal is matching
                %   to the copy of the original signal. Then matches those
                %   locations to the corresponding values from the signal
                %   matrix to the signal_copy matrix.
                signal(bad_indices == 1,j) = NaN;
                [~, index] = ismember(signal(:,1),signal_copy(:,1));
                signal_copy(index,j) = signal(:,j);
            end
            
        end
    end
        %   Output the signal        
        output_signal = signal_copy;

  %% EXAMPLE
  %The electric field data should have the first few and last few seconds
  %removed due to instrumental glitches
  
% test =cdfread('C:\Documents and Settings\vanwelie\Desktop\FastData\fa_k0_02217\fa_k0_02217_mag', 'CombineRecords', true);      
% test2 = [test{:,2:5}];
% % test2 = [test{:,2} test{:,3} test{:,4}];
% index = find(~isnan(test2(:,2)), 1, 'last');
% test2(test2(:,1) >= (test2(index,1)-7),2:3) = NaN;
% index = find(~isnan(test2(:,2)), 1, 'first');
% test2(test2(:,1) <= (test2(index,1)+6),2:3) = NaN;
% out= outlier_removal(test2,1000,'3',5);
% plot(test2(:,1),test2(:,3))
% hold on
% plot(out(:,1),out(:,3),'g');
            
end



