function [ef_radar] = radar_locate(orb_data, ef_data)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Megan Van Welie
% Worcester Polytechnic Institute
% Silicon Valley MQP 2011
% at SRI International
% megan6459@gmail.com
%
% This function limits electric field data that has  been limited down to 
% within +/- 1 degree of the invariant latitude, and +/- 2.5 degrees of the 
% longitude of the location of the Sondrestrom radar. This will be used by
% radar_locate_times.m to compile a list of orbit numbers, start times, and 
% end times of FAST orbits, which overlap the position of the Sondrestrom 
% radar.
%
% Inputs:
% orb_data - orbital data
% ef_data - electric field data
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % Hard coded latitude and longitude of the Sondrestrom radar
    radar_lat = 67;
    radar_lng = 309;
    
    % Limits within 1 (lat) and 2.5 (lng) degrees of the radar
    radar_lat_min = radar_lat - 1;
    radar_lat_max = radar_lat + 1;
    radar_lng_min = radar_lng - 2.5;
    radar_lng_max = radar_lng + 2.5;

    % Limit input orbital matrix to only required information
    orb_all = [orb_data{:,1}, orb_data{:,13:14}];
    %orb_all(:,1)       = unix time
    %orb_all(:,2)       = lat FOOT
    %orb_all(:,3)       = lng FOOT
    clearvars orb_data
    
    % Limit input electric field matrix to only required information
    ezero = zeros(length(ef_data{:,1}),1);
    ef_all = [ef_data{:,1}, ef_data{:,3}, ezero, ef_data{:,2}];
    %ef_all(:,1)        = unix time
    %ef_all(:,2:4)      = electric field x,y,z (DCS2)
    clearvars ef_data ezero
    
    % Interpolate positional data to the size of the electric field data
    [interp_orb interp_ef] = data_interp(orb_all, ef_all);
    
    % Create new matrix with electric field and positional data together
    ef_data = [interp_ef interp_orb(:,2:3)];
    %ef_data(:,1) = unix time
    %ef_data(:,2:4) = electric field x,y,z (DCS2)
    %ef_data(:,5:6) = lat FOOT, lng FOOT (deg)
    clearvars orb_all ef_all
    
    
    ef_lat = ef_data(:,5);
    ef_lng = ef_data(:,6)+180; %Convert longitude from -180 to 180 to 0 to 360 degrees
    
    % Limit electric field data down to overlap of position of radar
    ef_radar = ef_data((((radar_lat_min <= ef_lat) & (ef_lat <= radar_lat_max))&((radar_lng_min <= ef_lng) & (ef_lng <= radar_lng_max))),:);
    
end
